@@ -4,7 +4,11 @@ import sys
 import time
 
 import pytest
-from pathlib2 import Path
+
+if sys.version_info[:2] >= (3, 4):
+    from pathlib import Path
+else:
+    from pathlib2 import Path
 
 from tox.constants import INFO
 from tox.util.main import MAIN_FILE
@@ -66,7 +70,8 @@ def test_provision_from_pyvenv(initproj, cmd, monkeypatch):
 
 @pytest.mark.skipif(INFO.IS_PYPY, reason="TODO: process numbers work differently on pypy")
 @pytest.mark.skipif(
-    "sys.platform == 'win32'", reason="triggering SIGINT reliably on Windows is hard",
+    "sys.platform == 'win32'",
+    reason="triggering SIGINT reliably on Windows is hard",
 )
 def test_provision_interrupt_child(initproj, monkeypatch, capfd):
     monkeypatch.delenv(str("PYTHONPATH"), raising=False)
@@ -125,5 +130,6 @@ def test_provision_interrupt_child(initproj, monkeypatch, capfd):
 
     for process in all_process:
         assert not process.is_running(), "{}{}".format(
-            out, "\n".join(repr(i) for i in all_process),
+            out,
+            "\n".join(repr(i) for i in all_process),
         )
