@@ -3,6 +3,7 @@ import sys
 
 import py
 import pytest
+from six import PY2
 
 import tox
 from tox.interpreters import NoInterpreterInfo
@@ -83,16 +84,18 @@ def test_create(mocksession, newconfig):
     assert venv.path_config.check(exists=False)
 
 
-def test_create_KeyboardInterrupt(mocksession, newconfig, mocker):
+@pytest.mark.parametrize("patched_venv_methodname", ["_pcall", "update"])
+def test_create_KeyboardInterrupt(mocksession, newconfig, mocker, patched_venv_methodname):
     config = newconfig(
         [],
         """\
         [testenv:py123]
+        deps = pip >= 19.3.1
         """,
     )
     mocksession.new_config(config)
     venv = mocksession.getvenv("py123")
-    mocker.patch.object(venv, "_pcall", side_effect=KeyboardInterrupt)
+    mocker.patch.object(venv, patched_venv_methodname, side_effect=KeyboardInterrupt)
     with pytest.raises(KeyboardInterrupt):
         venv.setupenv()
 
@@ -450,6 +453,41 @@ def test_install_command_whitelisted(newmocksession):
     assert venv.status == "commands failed"
 
 
+def test_install_command_allowlisted(newmocksession):
+    mocksession = newmocksession(
+        ["--recreate"],
+        """\
+        [testenv]
+        allowlist_externals = pytest
+                              xy*
+        commands=
+            pytest
+            xyz
+        """,
+    )
+    venv = mocksession.getvenv("python")
+    venv.test()
+    mocksession.report.expect("warning", "*test command found but not*", invert=True)
+    assert venv.status == "commands failed"
+
+
+def test_install_command_allowlisted_exclusive(newmocksession):
+    mocksession = newmocksession(
+        ["--recreate"],
+        """\
+        [testenv]
+        allowlist_externals = pytest
+        whitelist_externals = xy*
+        commands=
+            pytest
+            xyz
+        """,
+    )
+    venv = mocksession.getvenv("python")
+    with pytest.raises(tox.exception.ConfigError):
+        venv.test()
+
+
 def test_install_command_not_installed_bash(newmocksession):
     mocksession = newmocksession(
         ["--recreate"],
@@ -735,21 +773,36 @@ class TestVenvTest:
         assert "PYTHONPATH" not in pcalls[0].env
 
 
-def test_env_variables_added_to_pcall(tmpdir, mocksession, newconfig, monkeypatch):
+def test_env_variables_added_to_pcall(tmpdir, mocksession, newconfig, monkeypatch, tmp_path):
     monkeypatch.delenv("PYTHONPATH", raising=False)
     pkg = tmpdir.ensure("package.tar.gz")
     monkeypatch.setenv("X123", "123")
     monkeypatch.setenv("YY", "456")
+    env_path = tmp_path / ".env"
+    env_file_content = "ENV_FILE_VAR = file_value"
+    env_path.write_text(env_file_content.decode() if PY2 else env_file_content)
+
     config = newconfig(
         [],
-        """\
+        r"""
+        [base]
+        base_var = base_value
+
         [testenv:python]
         commands=python -V
         passenv = x123
         setenv =
             ENV_VAR = value
+            ESCAPED_VAR = \{value\}
+            ESCAPED_VAR2 = \\{value\\}
+            BASE_VAR = {[base]base_var}
             PYTHONPATH = value
-        """,
+            TTY_VAR = {tty:ON_VALUE:OFF_VALUE}
+            COLON = {:}
+            REUSED_FILE_VAR = reused {env:ENV_FILE_VAR}
+            file| %s
+        """
+        % env_path,
     )
     mocksession._clearmocks()
     mocksession.new_config(config)
@@ -764,10 +817,18 @@ def test_env_variables_added_to_pcall(tmpdir, mocksession, newconfig, monkeypatc
         assert env is not None
         assert "ENV_VAR" in env
         assert env["ENV_VAR"] == "value"
+        assert env["ESCAPED_VAR"] == "{value}"
+        assert env["ESCAPED_VAR2"] == r"\{value\}"
+        assert env["COLON"] == ";" if sys.platform == "win32" else ":"
+        assert env["TTY_VAR"] == "OFF_VALUE"
+        assert env["ENV_FILE_VAR"] == "file_value"
+        assert env["REUSED_FILE_VAR"] == "reused file_value"
+        assert env["BASE_VAR"] == "base_value"
         assert env["VIRTUAL_ENV"] == str(venv.path)
         assert env["X123"] == "123"
         assert "PYTHONPATH" in env
         assert env["PYTHONPATH"] == "value"
+
     # all env variables are passed for installation
     assert pcalls[0].env["YY"] == "456"
     assert "YY" not in pcalls[1].env
@@ -857,6 +918,20 @@ def test_run_custom_install_command(newmocksession):
     assert pcalls[0].args[1:] == ["whatever"]
 
 
+def test_run_install_command_handles_KeyboardInterrupt(newmocksession, mocker):
+    mocksession = newmocksession([], "")
+    venv = mocksession.getvenv("python")
+    venv.just_created = True
+    venv.envconfig.envdir.ensure(dir=1)
+
+    mocker.patch.object(venv, "_pcall", side_effect=KeyboardInterrupt)
+    with mocksession.newaction(venv.name, "hello") as action:
+        with pytest.raises(KeyboardInterrupt):
+            venv.run_install_command(packages=["whatever"], action=action)
+
+    assert venv.status == "keyboardinterrupt"
+
+
 def test_command_relative_issue36(newmocksession, tmpdir, monkeypatch):
     mocksession = newmocksession(
         [],
